﻿#!/usr/bin/python
# -*- coding: utf-8 -*-

import os, os.path, fnmatch, imp, tarfile, zipfile, shutil

def file_decompress(file_in, path_out) :
  ''' decompress zip, tar, gz, bzip2 archive to destination folder '''
  ''' operation result 0 - Ok, -1 - decompress failed, -2 - file not exists '''
  result = -1
  if os.path.exists(file_in) :
    # normalize output path
    path_out = os.path.abspath(path_out)
    # if needed, create output folder
    if not os.path.exists(path_out) :
      os.makedirs(path_out)
    # if file is zip archive
    if (os.path.splitext(file_in)[1].lower() == '.zip') or (zipfile.is_zipfile(file_in)) :
      try :
        zf = zipfile.ZipFile(file_in)
        zi = zf.namelist()
        for zii in zi :
          if (len(zii) > 0) and (zii[-1] != '/') :
            fr = zf.read(zii) # read file to string
            fo = os.path.join(path_out, zii) # full output file path
            if not os.path.exists(os.path.split(fo)[0]) : os.makedirs(os.path.split(fo)[0]) # create output folders
            open(fo, 'wb').write(fr) # write file
        zf.close()
        result = 0
      except : pass
    # if file is tar/gz/bz2 archive
    elif (os.path.splitext(file_in)[1].lower() in ['.tar', '.gz', '.tgz', '.bz2', '.bzip2']) or (tarfile.is_tarfile(file_in)) :
      try :
        tarfile.TarFile.open(file_in, mode = 'r').extractall(path_out)
        result = 0
      except : print 'extraction error'
  else : # file not exists
    result = -2
  return result

def folder_compress(path_in, file_out, compression=None) :
  ''' recursively compress folder, compression can be '' (uncompressed tar), 'gz', 'bz2', 'zip' '''
  ''' returns 0 if all Ok, -1 - compression error, -2 - input path not exists '''
  result = -1
  # normalize input path
  path_in = os.path.abspath(path_in)
  # normalize output file path
  file_out = os.path.abspath(file_out)
  # estimate output file format
  if compression == None :
    if '.zip' in file_out.lower() : compression = 'zip'
    elif '.gz' in file_out.lower() : compression = 'gz'
    elif '.bz2' in file_out.lower() : compression = 'bz2'
    elif '.tar' in file_out.lower() : compression = ''
  if os.path.exists(path_in) :
    # remember current directory
    path_cur = os.path.abspath(os.path.abspath(os.curdir))
    # go to src folder
    os.chdir(path_in)
    # if necessary, create output path
    if not os.path.exists(os.path.split(file_out)[0]) :
      os.makedirs(os.path.split(file_out)[0])
    if compression == 'zip' : file_ext = '.zip'
    else :
      file_ext = '.tar'
      if compression != '' : file_ext += '.' + compression.lower()
    # if needed, add file extension
    if not file_ext in file_out : file_out += file_ext
    # create tar archive
    if '.tar' in file_ext :
      # compressed file extention
      if compression != '' :
        compression = ':' + compression
      out_file = tarfile.open(file_out, mode = 'w' + compression)
      #print file_out
      # compress input folder
      out_file.add(os.curdir, '')
      # add every file to archive
      #for root, dirs, files in os.walk(path_in) :
      #  for file_in in files :
      #    out_file.add(os.path.join(path_rel(root, path_in), file_in))#, os.path.join(path_rel(root, path_in), file_in))
      #    print os.path.join(path_rel(root, path_in), file_in)
      out_file.close()
      result = 0
    # create zip archive
    elif '.zip' in file_ext :
      try :
        # create zip archive
        zf = zipfile.ZipFile(file_out, mode = 'w', compression = zipfile.ZIP_DEFLATED)
        # add every file to archive
        for root, dirs, files in os.walk(path_in) :
          for file_in in files :
            zf.write(os.path.join(path_rel(root, path_in), file_in))
        zf.close()
        result = 0
      except : pass
    # return back to current directory
    os.chdir(path_cur)
  else :
    result = -2
  return result
